<?php

namespace Itgro\SbrfAcquiring\Controllers\Http;

use Artisan;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller as BaseController;
use Itgro\SbrfAcquiring\Client\Client;
use Itgro\SbrfAcquiring\Exceptions\OrderDoesNotExist;
use Itgro\SbrfAcquiring\Facades\ClientFacade;
use Itgro\SbrfAcquiring\Models\SbrfAcquiringLog;
use Itgro\SbrfAcquiring\Models\SbrfAcquiringOrder;
use Symfony\Component\Routing\Exception\InvalidParameterException;

class Pages extends BaseController
{

    public function payment(Request $request)
    {
        $amount = $request->input('amount');

        if (!$amount) {
            return redirect()->back()->withErrors([
                'amount_error' => 'Укажите сумму платежа',
            ]);
        }

        $order_id = $request->input('order_id', null);

        $parameters = $request->input('parameters', []);

        $result = config('sbrf-acquiring.preauth_payment') ?
            ClientFacade::registerPreauthOrder($amount, $order_id, $parameters) :
            ClientFacade::registerOrder($amount, $order_id, $parameters);

        if ($result) {
            return redirect()->to($result->formUrl);
        }

        return redirect()->route('sbrf-acquiring.fail')->with('error', $result->errorMessage);
    }

    public function success(Request $request)
    {
        $external_id = $request->input('orderId');

        $order = SbrfAcquiringOrder::where('external_id', $external_id)->first();

        if ($order) {
            Artisan::call('sbrf-acquiring:orders:check-state', [
                'order_id' => $order->id,
            ]);
        }

        return view('sbrf-acquiring::pages.success');
    }

    public function fail()
    {
        return view('sbrf-acquiring::pages.fail');
    }

    public function callback(Request $request)
    {
        $operation = $request->input('operation');

        $external_id = $request->input('mdOrder');

        if (!$external_id) {
            throw new InvalidParameterException();
        }

        $order = SbrfAcquiringOrder::where('external_id', $external_id)->first();

        if (!$order) {
            throw new OrderDoesNotExist('Платеж не зарегистрирован в системе.');
        }

        switch ($operation) {
            case SbrfAcquiringOrder::STATUS_DEPOSITED:
                Artisan::call('sbrf-acquiring:orders:check-state', [
                    'order_id' => $order->id,
                ]);
                break;

            case SbrfAcquiringOrder::STATUS_APPROVED:
            case SbrfAcquiringOrder::STATUS_REFUNDED:
            case SbrfAcquiringOrder::STATUS_REVERSED:
            default:
                break;
        }

        $log = new SbrfAcquiringLog;

        $log->type = 'info';
        $log->method = 'callbackPayment.' . $operation;
        $log->request = $request->all();
        $log->responce = 'ОК';

        $log->save();

        return 'OK';
    }

}