<?php

namespace Itgro\SbrfAcquiring;

use Illuminate\Foundation\Support\Providers\EventServiceProvider as ServiceProvider;
use Itgro\SbrfAcquiring\Events\OrderCreateFail as OrderCreateFailEvent;
use Itgro\SbrfAcquiring\Events\OrderCreateSuccess as OrderCreateSuccessEvent;
use Itgro\SbrfAcquiring\Events\OrderCreateSuccess;
use Itgro\SbrfAcquiring\Events\Payments\ {
    CheckState as PaymentCheckStateEvent,
    Declined as PaymentDeclinedEvent,
    Deposited as PaymentDepositedEvent,
    Fail as PaymentFailEvent,
    Refunded as PaymentRefundedEvent,
    Reversed as PaymentReversedEvent,
    Success as PaymentSuccessEvent,
};
use Itgro\SbrfAcquiring\Listeners\Payments\ {
    Accept as PaymentAcceptListener,
};
use Itgro\SbrfAcquiring\Listeners\Order\Created as OrderCreatedListener;

use Itgro\SbrfAcquiring\Events\Client\ {
    Executed as ClientExecutedEvent,
    ExecuteError as ClientExecuteErrorEvent,
};

use Itgro\SbrfAcquiring\Listeners\Client\ {
    Executed as ClientExecutedListener,
    ExecuteError as ClientExecuteErrorListener,
    ExecuteLog as ClientExecuteLogListener,
};

use Itgro\SbrfAcquiring\Models\SbrfAcquiringOrder;

class EventServiceProvider extends ServiceProvider
{
    protected $listen = [
        ClientExecutedEvent::class => [
            ClientExecutedListener::class,
            ClientExecuteLogListener::class,
        ],
        ClientExecuteErrorEvent::class => [
            ClientExecuteErrorListener::class,
            ClientExecuteLogListener::class,
        ],
        PaymentCheckStateEvent::class => [],
        PaymentDeclinedEvent::class => [],

        PaymentDepositedEvent::class => [
            PaymentAcceptListener::class,
        ],

        PaymentFailEvent::class => [],
        PaymentRefundedEvent::class => [],
        PaymentReversedEvent::class => [],
        PaymentSuccessEvent::class => [],
        OrderCreateFailEvent::class => [],

        OrderCreateSuccessEvent::class => [
            OrderCreatedListener::class,
        ],
    ];

    private $listenPaymentState = [
        SbrfAcquiringOrder::STATUS_APPROVED => PaymentDepositedEvent::class,
        SbrfAcquiringOrder::STATUS_DEPOSITED => PaymentSuccessEvent::class,
        SbrfAcquiringOrder::STATUS_REVERSED => PaymentReversedEvent::class,
        SbrfAcquiringOrder::STATUS_REFUNDED => PaymentRefundedEvent::class,
    ];

    public function boot()
    {
        parent::boot();

        $this->registerListenPaymentState();
    }

    private function registerListenPaymentState()
    {
        SbrfAcquiringOrder::saving(function (SbrfAcquiringOrder $order) {
            if ($order->status === $order->getOriginal('status')) {
                return;
            }

            $event = array_get($this->listenPaymentState, $order->status);

            if ($event) {
                event(new $event($order));
            }
        });
    }

}
